package top.ninwoo.cloud.service.cpcTrans;

import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import top.ninwoo.api.CnfData;
import top.ninwoo.api.Picture;
import top.ninwoo.cloud.service.TransService;
import top.ninwoo.cloud.service.bpso.Bpso;
import top.ninwoo.cloud.service.findService.FindServiceImpl;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentHashMap;

import static top.ninwoo.cloud.service.bpso.Bpso.lastHop;

@Service
public class CpcTrans implements TransService {

    @Resource
    private RestTemplate restTemplate;
    /**
     * 打包picture
     *
     * @return picture列表
     */
    @Override
    public Picture packagePic(BufferedImage bufferedImage, String fileId){
        Picture picture = new Picture();
        //转化为byte[]数组
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            //todo 需要增加输入图片格式
            ImageIO.write(bufferedImage, "jpg", out);
        } catch (IOException e) {
            e.printStackTrace();
        }
        //设置图片数据
        picture.setData(out.toByteArray());
        //设置文件名
        picture.setPicId(fileId);
        return picture;
    }

    /**
     * 打包成数据包
     */
    @Override
    public LinkedList<CnfData> packageData(Picture picture) throws ParseException {
        LinkedList<CnfData> cnfDatas = new LinkedList<>();
        FindServiceImpl findService = new FindServiceImpl();
        //判断npe
        if(picture == null){
            System.out.println("出错");
            return null;
        }

        //      输入：
        //子任务依赖关系 边
        int[][] link = {{0, 1, 1, 2, 3, 4, 5, 6}, {1, 2, 3, 4, 5, 6, 6, 7}};
        //task编号
        int[] func = {0, 2, 3, 3, 1, 1, 4, 0};
        //任务表
        String[] tasks = {"start/end","binaryProcess", "cutProcess", "greyProcess", "mergeProcess", "tailorProcess"};

        Bpso doMap = new Bpso();
        doMap.doMapApp(link,func);
        //映射结果
        int[] result = doMap.result;
        System.out.println("optimal mapping scheme"+ Arrays.toString(result));

        //路径表
        String[] path = new String[link[0].length];
        for (int tm = 0;tm<link[0].length;tm++){
            String startHop = "sate"+result[link[0][tm]];
            String endHop = "sate"+result[link[1][tm]];
            if(startHop.equals("sate0")){
                path[tm] = "D1" +"->"+ endHop;
                continue;
            }
            //要修改
            //todo
            if(endHop.equals("sate31")){
                path[tm] = startHop +"->"+ "D2";
                continue;
            }
            //左为边起点，右为边终点
            path[tm] = startHop+"->"+endHop;
        }

        //任务表
        ConcurrentHashMap<String,String> taskTable = new ConcurrentHashMap();
        for (int tm = 0;tm<result.length;tm++){
            String hop = "sate"+result[tm];
            if(hop.equals("sate0")){
                taskTable.put("D1",tasks[func[tm]]);
                continue;
            }
            //要修改
            //todo
            if(hop.equals("sate31")){
                taskTable.put("D2",tasks[func[tm]]);
                continue;
            }
            taskTable.put(hop,tasks[func[tm]]);
        }
        //T3路由表
        String[][] route = new String[lastHop.length][lastHop[0].length];
        for(int i =0;i<lastHop.length;i++){
            for (int j=0;j<lastHop[0].length;j++){
                route[i][j] = "sate"+(lastHop[i][j]);
                if (route[i][j].equals("sate0")){
                    route[i][j]="sate"+(j);
                }
                if (route[i][j].equals("sate0")){
                    route[i][j]="D1";
                }
                //要修改
                //todo
                if (route[i][j].equals("sate31")){
                    route[i][j]="D2";
                }
            }
        }
        boolean flad = false;
        for(int i =0;i<lastHop.length;i++){
            for (int j=0;j<lastHop[0].length;j++){
                if (route[i][j].equals("sate"+(i)) && i!=j){
                    flad = true;
                }
            }
        }
        if (flad==false){
            System.out.println("correct");
        }

        CnfData dataPackage = new CnfData();
        ConcurrentHashMap<String,String> ipool = findService.findIPool();
        dataPackage.setIpPool(ipool);

        //路径表
        dataPackage.setPathTable(path);
        //任务表
        dataPackage.setTaskTable(taskTable);
        //路由表
        dataPackage.setRoute(route);
        //图片
        dataPackage.setPicture(picture);
        //ip池
        dataPackage.setIpPool(findService.findIPool());
        String[] routeself = findService.findRoute(dataPackage.getCurrentSataHop(), dataPackage.getRoute());
        ArrayList<String> destinSataHops = findService.findDestinSataHops(path,"D1");
        for(String des : destinSataHops) {
            CnfData transferPackage = dataPackage;
            //目的
            transferPackage.setDestinSataHop(des);
            //当前
            transferPackage.setNextSataHop(findService.findNextHop(des, routeself));
            //下一跳
            transferPackage.setCurrentSataHop(transferPackage.getNextSataHop());
            //ip
            transferPackage.setNextIPHop(findService.findIp(transferPackage.getNextSataHop(),transferPackage.getIpPool()));
            //终止节点
            transferPackage.setEndSateHop("sate11");
            //返回一个数据包
            cnfDatas.add(transferPackage);
        }
        return cnfDatas;
    }

    @Override
    public LinkedList<CnfData> packageData(Picture picture, String satename) throws ParseException {
        return null;
    }

    /**
     * 发送数据包
     *
     * @param cnfData
     * @param url
     */
    @Override
    public String sendData(CnfData cnfData, String url) {
        /*Map<String, Object> param = new HashMap<>();
        param.put("cnfData ", cnfData);*/
        String IP = url + ":8083";
        ResponseEntity<String> res = restTemplate.postForEntity("http://" + IP + "/transfer", cnfData, String.class);
        if (!res.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("send Error!");
        }
        return "sucess";
    }

    /**
     * 发送得到cpu计算能力的指令
     *
     * @param url
     */
    @Override
    public Double sendCpuCtr(String url) {
        String IP = url + ":8083";
        return restTemplate.getForObject("http://" + IP + "/getCpuComputing", Double.class);
    }

    /**
     * 得到稳态图
     */
    @Override
    public String[][] getIt() throws ParseException {
        return new String[0][];
    }


    private boolean repetition(int[] result) {
        HashSet<Integer> set = new HashSet<>();
        for (int a : result){
            set.add(a);
        }
        if (set.size()==result.length) return false;
        else return true;
    }
}

